<!-- start custom head snippets -->
<link rel="stylesheet" type="text/css" media="screen" href="{{'/assets/css/custom.css' | relative_url}}">
<link rel="shortcut icon" href="/assets/images/favicon.ico">
{% include vendors/google-tag-manager.html %}
{% include vendors/hotjar.html %}
<!-- end custom head snippets -->