<div class="query__box">
  <div class="query query--left">
    <h3>PyMongo</h3>
      {{ include.pymongo | markdownify }}
  </div>
  <div class="query query--right">
    <h3>Djongo</h3>
      {{ include.djongo | markdownify }}
  </div>
</div>