{% if page.pizza %}
  <aside class="sidebar__right" style="position: sticky; top: 2em; float: right">
    <header><h4 class="nav__title"><i class="fas fa-{{ page.toc_icon | default: 'file-alt' }}"></i> Buy ME A Pizza</h4></header>
    <a href="https://www.patreon.com/posts/buy-me-pizza-26095081"><img src="/assets/images/pizza.jpg"></a>
  </aside>
{% endif %}