
<section  itemscope itemtype="https://schema.org/Offer">
  <h1 itemprop="category">Support</h1>
  <div class="tire">
    {% for tire in site.data.support.tire_column.tires %}
      {% if tire.invisible != true %}
        <div class="tire-item">
          <div itemprop="name" class="tire-item__title">{{ tire.title }}</div>
          <div itemprop="price" class="tire-item__price">${{ tire.price }}</div>
          <div itemprop="eligibleDuration"  class="tire-item__period">
            {% if tire.per_hour %}
              PER HOUR
            {% else %}
              PER MONTH
            {% endif %}
          </div>
          <a class="btn btn--success btn--large" href="/support/contact/{{ tire.url }}">Apply</a>
          <div itemprop="description">
            <ul>
              {% for b in tire.benefits %}
                <li>{{ b | markdownify | remove: "<p>" | remove: "</p>" }}</li>
              {% endfor %}
            </ul>
          </div>
        </div>
      {% endif %}
    {% endfor %}
  </div>
</section>
<section itemscope itemtype="https://schema.org/Offer">
  <h1 itemprop="category">Enterprise</h1>
  <div class="enterprise">
    <div itemprop="description" class="enterprise__text-box">
      {{ include.enterprise | markdownify }}
    </div>
    {% include form.html form=site.data.support.enterprise_form %}
  </div>
</section>

<section>
  {{ include.discuss | markdownify }}
</section>
<section>
  <h1 id="contact-us">Get In Touch</h1>
  {% include form.html form=site.data.support.contact_form %}
</section>



